package com.zanxiang.game.back.serve.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.advertising.tencent.base.AdvertisingTencentServer;
import com.zanxiang.advertising.tencent.base.rpc.IUserActionSetRpc;
import com.zanxiang.erp.base.ErpServer;
import com.zanxiang.erp.base.rpc.ISysUserRpc;
import com.zanxiang.game.back.base.pojo.enums.OrderStatusEnum;
import com.zanxiang.game.back.serve.dao.mapper.GameTencentMiniGameOrderMapper;
import com.zanxiang.game.back.serve.pojo.dto.GameTencentMiniGameOrderDTO;
import com.zanxiang.game.back.serve.pojo.entity.GameBackPolicy;
import com.zanxiang.game.back.serve.pojo.entity.GameTencentMiniGameOrder;
import com.zanxiang.game.back.serve.pojo.enums.BackStatusEnum;
import com.zanxiang.game.back.serve.pojo.vo.GameTencentMiniGameOrderVO;
import com.zanxiang.game.back.serve.service.IGameBackPolicyService;
import com.zanxiang.game.back.serve.service.IGameTencentMiniGameBackLogService;
import com.zanxiang.game.back.serve.service.IGameTencentMiniGameOrderService;
import com.zanxiang.game.back.serve.utils.BackPolicyUtil;
import com.zanxiang.module.util.bean.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.DubboReference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;

@Slf4j
@Service
public class GameTencentMiniGameOrderServiceImpl extends ServiceImpl<GameTencentMiniGameOrderMapper, GameTencentMiniGameOrder>
        implements IGameTencentMiniGameOrderService {

    @DubboReference(providedBy = AdvertisingTencentServer.SERVER_DUBBO_NAME)
    private IUserActionSetRpc userActionSetRpc;
    @DubboReference(providedBy = ErpServer.SERVER_DUBBO_NAME)
    private ISysUserRpc sysUserRpc;
    @Autowired
    private IGameTencentMiniGameBackLogService gameTencentMiniGameBackLogService;
    @Autowired
    private IGameBackPolicyService gameBackPolicyService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean orderBack(GameTencentMiniGameOrder orderLog, boolean mustBack) {
        boolean doBack = false;
        if (mustBack) {
            doBack = true;
        } else {
            if (!Objects.equals(orderLog.getOrderStatus(), OrderStatusEnum.SUCCESS_PAY.getValue()) &&
                    !Objects.equals(orderLog.getOrderStatus(), OrderStatusEnum.SUCCESS_PAY.getValue())) {
                // 只要回传 下单、支付行为
                return false;
            }
        }
        if (orderLog.getBackPolicyId() == null) {
            // 没有回传策略，则直接全量回传
            doBack = true;
        } else {
            GameBackPolicy gameBackPolicy = gameBackPolicyService.getById(orderLog.getBackPolicyId());
            doBack = BackPolicyUtil.backOrder(gameBackPolicy, orderLog.getRechargeMoney(),
                    () -> count(new LambdaQueryWrapper<GameTencentMiniGameOrder>()
                            .eq(GameTencentMiniGameOrder::getGameId, orderLog.getGameId())
                            .eq(GameTencentMiniGameOrder::getWechatAppId, orderLog.getWechatAppId())
                            .eq(GameTencentMiniGameOrder::getAdAccountId, orderLog.getAdAccountId())
                            .eq(GameTencentMiniGameOrder::getOrderStatus, OrderStatusEnum.SUCCESS_PAY.getValue())
                            .eq(GameTencentMiniGameOrder::getWechatOpenid, orderLog.getWechatOpenid())
                            .last("limit 1")
                    ) < 1, backUnit -> Long.valueOf(count(new LambdaQueryWrapper<GameTencentMiniGameOrder>()
                            .eq(GameTencentMiniGameOrder::getGameId, orderLog.getGameId())
                            .eq(GameTencentMiniGameOrder::getWechatAppId, orderLog.getWechatAppId())
                            .eq(GameTencentMiniGameOrder::getAdAccountId, orderLog.getAdAccountId())
                            .eq(GameTencentMiniGameOrder::getOrderStatus, OrderStatusEnum.SUCCESS_PAY.getValue())
                            .orderByDesc(GameTencentMiniGameOrder::getCreateTime)
                            .last("limit " + backUnit)
                    )).intValue());
            orderLog.setRechargeMoney(BackPolicyUtil.lowRechargeLevel(orderLog.getRechargeMoney(), gameBackPolicy.getLevelDown()));
        }
        if (doBack) {
            BackStatusEnum backStatus = gameTencentMiniGameBackLogService.orderBack(orderLog);
            update(new LambdaUpdateWrapper<GameTencentMiniGameOrder>()
                    .set(GameTencentMiniGameOrder::getBackStatus, backStatus.getBackStatus())
                    .eq(GameTencentMiniGameOrder::getId, orderLog.getId())
            );
            return true;
        }
        return false;
    }


    @Override
    public IPage<GameTencentMiniGameOrderVO> listOfPage(GameTencentMiniGameOrderDTO dto) {
        return page(dto.toPage(), new LambdaQueryWrapper<GameTencentMiniGameOrder>()
                .eq(StringUtils.isNotBlank(dto.getAgentKey()), GameTencentMiniGameOrder::getAgentKey, dto.getAgentKey())
                .eq(null != dto.getGameId(), GameTencentMiniGameOrder::getGameId, dto.getGameId())
                .eq(null != dto.getAdAccountId(), GameTencentMiniGameOrder::getAdAccountId, dto.getAdAccountId())
                .eq(StringUtils.isNotBlank(dto.getWechatAppId()), GameTencentMiniGameOrder::getWechatAppId, dto.getWechatAppId())
                .eq(StringUtils.isNotBlank(dto.getWechatOpenid()), GameTencentMiniGameOrder::getWechatOpenid, dto.getWechatOpenid())
                .eq(StringUtils.isNotBlank(dto.getOrderId()), GameTencentMiniGameOrder::getOrderId, dto.getOrderId())
                .eq(null != dto.getBackStatus(), GameTencentMiniGameOrder::getBackStatus, dto.getBackStatus())
                .eq(null != dto.getOrderStatus(), GameTencentMiniGameOrder::getOrderStatus, dto.getOrderStatus())
                .orderByDesc(GameTencentMiniGameOrder::getCreateTime)
        ).convert(this::toVOSimple);
    }

    @Override
    public boolean doReport(List<Long> orderLogIds) {
        listByIds(orderLogIds).stream()
                .filter(item -> !item.getBackStatus().equals(BackStatusEnum.SUCCESS.getBackStatus()))
                .forEach(orderLog -> gameTencentMiniGameBackLogService.orderBack(orderLog));
        return true;
    }

    private GameTencentMiniGameOrderVO toVOSimple(GameTencentMiniGameOrder orderLog) {
        return BeanUtil.copy(orderLog, GameTencentMiniGameOrderVO.class);
    }
}
